@@ -16,16 +16,6 @@ from oslo_config import cfg
 from oslo_messaging._drivers import common
 
 KAFKA_OPTS = [
-    cfg.StrOpt('kafka_default_host', default='localhost',
-               deprecated_for_removal=True,
-               deprecated_reason="Replaced by [DEFAULT]/transport_url",
-               help='Default Kafka broker Host'),
-
-    cfg.PortOpt('kafka_default_port', default=9092,
-                deprecated_for_removal=True,
-                deprecated_reason="Replaced by [DEFAULT]/transport_url",
-                help='Default Kafka broker Port'),
-
     cfg.IntOpt('kafka_max_fetch_bytes', default=1024 * 1024,
                help='Max fetch bytes of Kafka consumer'),
 
@@ -56,7 +46,27 @@ KAFKA_OPTS = [
                       "in seconds"),
 
     cfg.IntOpt('producer_batch_size', default=16384,
-               help='Size of batch for the producer async send')
+               help='Size of batch for the producer async send'),
+
+    cfg.BoolOpt('enable_auto_commit',
+                default=False,
+                help='Enable asynchronous consumer commits'),
+
+    cfg.IntOpt('max_poll_records', default=500,
+               help='The maximum number of records returned in a poll call'),
+
+    cfg.StrOpt('security_protocol', default='PLAINTEXT',
+               choices=('PLAINTEXT', 'SASL_PLAINTEXT', 'SSL', 'SASL_SSL'),
+               help='Protocol used to communicate with brokers'),
+
+    cfg.StrOpt('sasl_mechanism',
+               default='PLAIN',
+               help='Mechanism when security protocol is SASL'),
+
+    cfg.StrOpt('ssl_cafile',
+               default='',
+               help='CA certificate PEM file used to verify the server'
+               ' certificate')
 ]
 
 
