@@ -16,13 +16,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-__all__ = [
-    'ExecutorLoadFailure',
-    'MessageHandlingServer',
-    'MessagingServerError',
-    'ServerListenError',
-]
-
 import abc
 import functools
 import inspect
@@ -42,6 +35,13 @@ from oslo_messaging._drivers import base as driver_base
 from oslo_messaging._i18n import _LW
 from oslo_messaging import exceptions
 
+__all__ = [
+    'ExecutorLoadFailure',
+    'MessageHandlingServer',
+    'MessagingServerError',
+    'ServerListenError',
+]
+
 LOG = logging.getLogger(__name__)
 
 # The default number of seconds of waiting after which we will emit a log
@@ -395,13 +395,11 @@ class MessageHandlingServer(service.ServiceBase, _OrderedTaskRunner):
         choose to dispatch messages in a new thread, coroutine or simply the
         current thread.
         """
-        # Warn that restarting will be deprecated
         if self._started:
-            LOG.warning(_LW('Restarting a MessageHandlingServer is inherently '
-                            'racy. It is deprecated, and will become a noop '
-                            'in a future release of oslo.messaging. If you '
-                            'need to restart MessageHandlingServer you should '
-                            'instantiate a new object.'))
+            LOG.warning(_LW('The server has already been started. Ignoring'
+                            ' the redundant call to start().'))
+            return
+
         self._started = True
 
         executor_opts = {}
@@ -428,7 +426,8 @@ class MessageHandlingServer(service.ServiceBase, _OrderedTaskRunner):
         some messages, and underlying driver resources associated to this
         server are still in use. See 'wait' for more details.
         """
-        self.listener.stop()
+        if self.listener:
+            self.listener.stop()
         self._started = False
 
     @ordered(after='stop')
@@ -445,7 +444,8 @@ class MessageHandlingServer(service.ServiceBase, _OrderedTaskRunner):
         self._work_executor.shutdown(wait=True)
 
         # Close listener connection after processing all messages
-        self.listener.cleanup()
+        if self.listener:
+            self.listener.cleanup()
 
     def reset(self):
         """Reset service.
