@@ -22,7 +22,6 @@ import uuid
 import fixtures
 import kombu
 import kombu.transport.memory
-from oslo_config import cfg
 from oslo_serialization import jsonutils
 import testscenarios
 
@@ -37,24 +36,6 @@ from six.moves import mock
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
-class TestDeprecatedRabbitDriverLoad(test_utils.BaseTestCase):
-
-    def setUp(self):
-        super(TestDeprecatedRabbitDriverLoad, self).setUp(
-            conf=cfg.ConfigOpts())
-        self.messaging_conf.transport_driver = 'rabbit'
-        self.config(fake_rabbit=True, group="oslo_messaging_rabbit")
-
-    def test_driver_load(self):
-        transport = oslo_messaging.get_transport(self.conf)
-        self.addCleanup(transport.cleanup)
-        driver = transport._driver
-        url = driver._get_connection()._url
-
-        self.assertIsInstance(driver, rabbit_driver.RabbitDriver)
-        self.assertEqual('memory:////', url)
-
-
 class TestHeartbeat(test_utils.BaseTestCase):
 
     @mock.patch('oslo_messaging._drivers.impl_rabbit.LOG')
@@ -137,27 +118,23 @@ class TestRabbitQos(test_utils.BaseTestCase):
 class TestRabbitDriverLoad(test_utils.BaseTestCase):
 
     scenarios = [
-        ('rabbit', dict(transport_driver='rabbit',
-                        url='amqp://guest:guest@localhost:5672//')),
-        ('kombu', dict(transport_driver='kombu',
-                       url='amqp://guest:guest@localhost:5672//')),
-        ('rabbit+memory', dict(transport_driver='kombu+memory',
-                               url='memory:///'))
+        ('rabbit', dict(transport_url='rabbit:/guest:guest@localhost:5672//')),
+        ('kombu', dict(transport_url='kombu:/guest:guest@localhost:5672//')),
+        ('rabbit+memory', dict(transport_url='kombu+memory:/'))
     ]
 
-    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
+                '.ensure_connection')
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
     def test_driver_load(self, fake_ensure, fake_reset):
         self.config(heartbeat_timeout_threshold=60,
                     group='oslo_messaging_rabbit')
-        self.messaging_conf.transport_driver = self.transport_driver
+        self.messaging_conf.transport_url = self.transport_url
         transport = oslo_messaging.get_transport(self.conf)
         self.addCleanup(transport.cleanup)
         driver = transport._driver
-        url = driver._get_connection()._url
 
         self.assertIsInstance(driver, rabbit_driver.RabbitDriver)
-        self.assertEqual(self.url, url)
 
 
 class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
@@ -179,7 +156,8 @@ class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
                                                 cert_reqs=ssl.CERT_REQUIRED))),
     ]
 
-    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
+                '.ensure_connection')
     @mock.patch('kombu.connection.Connection')
     def test_driver_load(self, connection_klass, fake_ensure):
         self.config(group="oslo_messaging_rabbit", **self.options)
@@ -389,7 +367,7 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
 
     scenarios = [
         ('none', dict(url=None,
-                      expected=["amqp://guest:guest@localhost:5672//"])),
+                      expected=["amqp://guest:guest@localhost:5672/"])),
         ('memory', dict(url='kombu+memory:////',
                         expected=["memory:///"])),
         ('empty',
@@ -422,15 +400,19 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
         ('rabbit_ipv4',
          dict(url='rabbit://user:password@10.20.30.40:10/vhost',
               expected=['amqp://user:password@10.20.30.40:10/vhost'])),
+        ('rabbit_no_vhost_slash',
+         dict(url='rabbit://user:password@10.20.30.40:10',
+              expected=['amqp://user:password@10.20.30.40:10/'])),
     ]
 
     def setUp(self):
         super(TestRabbitTransportURL, self).setUp()
-        self.messaging_conf.transport_driver = 'rabbit'
+        self.messaging_conf.transport_url = 'rabbit:/'
         self.config(heartbeat_timeout_threshold=0,
                     group='oslo_messaging_rabbit')
 
-    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
+    @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection'
+                '.ensure_connection')
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
     def test_transport_url(self, fake_reset, fake_ensure):
         transport = oslo_messaging.get_transport(self.conf, self.url)
@@ -469,8 +451,11 @@ class TestSendReceive(test_utils.BaseTestCase):
     ]
 
     _timeout = [
-        ('no_timeout', dict(timeout=None)),
-        ('timeout', dict(timeout=0.01)),  # FIXME(markmc): timeout=0 is broken?
+        ('no_timeout', dict(timeout=None, call_monitor_timeout=None)),
+        ('timeout', dict(timeout=0.01,  # FIXME(markmc): timeout=0 is broken?
+                         call_monitor_timeout=None)),
+        ('call_monitor_timeout', dict(timeout=0.01,
+                                      call_monitor_timeout=0.02)),
     ]
 
     @classmethod
@@ -500,15 +485,20 @@ class TestSendReceive(test_utils.BaseTestCase):
         replies = []
         msgs = []
 
+        # FIXME(danms): Surely this is not the right way to do this...
+        self.ctxt['client_timeout'] = self.call_monitor_timeout
+
         def send_and_wait_for_reply(i):
             try:
 
                 timeout = self.timeout
+                cm_timeout = self.call_monitor_timeout
                 replies.append(driver.send(target,
                                            self.ctxt,
                                            {'tx_id': i},
                                            wait_for_reply=True,
-                                           timeout=timeout))
+                                           timeout=timeout,
+                                           call_monitor_timeout=cm_timeout))
                 self.assertFalse(self.failure)
                 self.assertIsNone(self.timeout)
             except (ZeroDivisionError, oslo_messaging.MessagingTimeout) as e:
@@ -594,14 +584,15 @@ class TestRacyWaitForReply(test_utils.BaseTestCase):
         wait_conditions = []
         orig_reply_waiter = amqpdriver.ReplyWaiter.wait
 
-        def reply_waiter(self, msg_id, timeout):
+        def reply_waiter(self, msg_id, timeout, call_monitor_timeout):
             if wait_conditions:
                 cond = wait_conditions.pop()
                 with cond:
                     cond.notify()
                 with cond:
                     cond.wait()
-            return orig_reply_waiter(self, msg_id, timeout)
+            return orig_reply_waiter(self, msg_id, timeout,
+                                     call_monitor_timeout)
 
         self.useFixture(fixtures.MockPatchObject(
             amqpdriver.ReplyWaiter, 'wait', reply_waiter))
@@ -891,11 +882,12 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
     ]
 
     _context = [
-        ('empty_ctxt', dict(ctxt={}, expected_ctxt={})),
+        ('empty_ctxt', dict(ctxt={}, expected_ctxt={'client_timeout': None})),
         ('user_project_ctxt',
          dict(ctxt={'_context_user': 'mark',
                     '_context_project': 'snarkybunch'},
-              expected_ctxt={'user': 'mark', 'project': 'snarkybunch'})),
+              expected_ctxt={'user': 'mark', 'project': 'snarkybunch',
+                             'client_timeout': None})),
     ]
 
     _compression = [
@@ -940,6 +932,7 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
             '_msg_id': uuid.uuid4().hex,
             '_unique_id': uuid.uuid4().hex,
             '_reply_q': 'reply_' + uuid.uuid4().hex,
+            '_timeout': None,
         })
 
         msg['oslo.message'] = jsonutils.dumps(msg['oslo.message'])
@@ -959,9 +952,9 @@ class RpcKombuHATestCase(test_utils.BaseTestCase):
 
     def setUp(self):
         super(RpcKombuHATestCase, self).setUp()
-        self.brokers = ['host1', 'host2', 'host3', 'host4', 'host5']
-        self.config(rabbit_hosts=self.brokers,
-                    rabbit_retry_interval=0.01,
+        transport_url = 'rabbit:/host1,host2,host3,host4,host5/'
+        self.messaging_conf.transport_url = transport_url
+        self.config(rabbit_retry_interval=0.01,
                     rabbit_retry_backoff=0.01,
                     kombu_reconnect_delay=0,
                     heartbeat_timeout_threshold=0,
