@@ -51,72 +51,49 @@ class _FakeManager(object):
 class GetTransportTestCase(test_utils.BaseTestCase):
 
     scenarios = [
-        ('rpc_backend',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
-              control_exchange=None, allowed=None, aliases=None,
-              expect=dict(backend='testbackend',
+        ('default',
+         dict(url=None, transport_url=None,
+              control_exchange=None, allowed=None,
+              expect=dict(backend='rabbit',
                           exchange=None,
-                          url='testbackend:',
+                          url='rabbit:',
                           allowed=[]))),
         ('transport_url',
-         dict(url=None, transport_url='testtransport:', rpc_backend=None,
-              control_exchange=None, allowed=None, aliases=None,
+         dict(url=None, transport_url='testtransport:',
+              control_exchange=None, allowed=None,
               expect=dict(backend='testtransport',
                           exchange=None,
                           url='testtransport:',
                           allowed=[]))),
         ('url_param',
-         dict(url='testtransport:', transport_url=None, rpc_backend=None,
-              control_exchange=None, allowed=None, aliases=None,
+         dict(url='testtransport:', transport_url=None,
+              control_exchange=None, allowed=None,
               expect=dict(backend='testtransport',
                           exchange=None,
                           url='testtransport:',
                           allowed=[]))),
         ('control_exchange',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
-              control_exchange='testexchange', allowed=None, aliases=None,
+         dict(url=None, transport_url='testbackend:',
+              control_exchange='testexchange', allowed=None,
               expect=dict(backend='testbackend',
                           exchange='testexchange',
                           url='testbackend:',
                           allowed=[]))),
         ('allowed_remote_exmods',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
-              control_exchange=None, allowed=['foo', 'bar'], aliases=None,
+         dict(url=None, transport_url='testbackend:',
+              control_exchange=None, allowed=['foo', 'bar'],
               expect=dict(backend='testbackend',
                           exchange=None,
                           url='testbackend:',
                           allowed=['foo', 'bar']))),
-        ('rpc_backend_aliased',
-         dict(url=None, transport_url=None, rpc_backend='testfoo',
-              control_exchange=None, allowed=None,
-              aliases=dict(testfoo='testbackend'),
-              expect=dict(backend='testbackend',
-                          exchange=None,
-                          url='testbackend:',
-                          allowed=[]))),
-        ('transport_url_aliased',
-         dict(url=None, transport_url='testfoo:', rpc_backend=None,
-              control_exchange=None, allowed=None,
-              aliases=dict(testfoo='testtransport'),
-              expect=dict(backend='testtransport',
-                          exchange=None,
-                          url='testtransport:',
-                          allowed=[]))),
-        ('url_param_aliased',
-         dict(url='testfoo:', transport_url=None, rpc_backend=None,
-              control_exchange=None, allowed=None,
-              aliases=dict(testfoo='testtransport'),
-              expect=dict(backend='testtransport',
-                          exchange=None,
-                          url='testtransport:',
-                          allowed=[]))),
     ]
 
     @mock.patch('oslo_messaging.transport.LOG')
     def test_get_transport(self, fake_logger):
-        self.config(rpc_backend=self.rpc_backend,
-                    control_exchange=self.control_exchange,
-                    transport_url=self.transport_url)
+        self.messaging_conf.reset()
+        self.config(control_exchange=self.control_exchange)
+        if self.transport_url:
+            self.config(transport_url=self.transport_url)
 
         driver.DriverManager = mock.Mock()
 
@@ -133,18 +110,8 @@ class GetTransportTestCase(test_utils.BaseTestCase):
         kwargs = dict(url=self.url)
         if self.allowed is not None:
             kwargs['allowed_remote_exmods'] = self.allowed
-        if self.aliases is not None:
-            kwargs['aliases'] = self.aliases
         transport_ = oslo_messaging.get_transport(self.conf, **kwargs)
 
-        if self.aliases is not None:
-            self.assertEqual(
-                [mock.call('legacy "rpc_backend" is deprecated, '
-                           '"testfoo" must be replaced by '
-                           '"%s"' % self.aliases.get('testfoo'))],
-                fake_logger.warning.mock_calls
-            )
-
         self.assertIsNotNone(transport_)
         self.assertIs(transport_.conf, self.conf)
         self.assertIs(transport_._driver, drvr)
@@ -161,57 +128,32 @@ class GetTransportSadPathTestCase(test_utils.BaseTestCase):
 
     scenarios = [
         ('invalid_transport_url',
-         dict(url=None, transport_url='invalid', rpc_backend=None,
+         dict(url=None, transport_url='invalid',
               ex=dict(cls=oslo_messaging.InvalidTransportURL,
                       msg_contains='No scheme specified',
                       url='invalid'))),
         ('invalid_url_param',
-         dict(url='invalid', transport_url=None, rpc_backend=None,
+         dict(url='invalid', transport_url=None,
               ex=dict(cls=oslo_messaging.InvalidTransportURL,
                       msg_contains='No scheme specified',
                       url='invalid'))),
         ('driver_load_failure',
-         dict(url=None, transport_url=None, rpc_backend='testbackend',
+         dict(url=None, transport_url='testbackend:/',
               ex=dict(cls=oslo_messaging.DriverLoadFailure,
                       msg_contains='Failed to load',
                       driver='testbackend'))),
     ]
 
     def test_get_transport_sad(self):
-        self.config(rpc_backend=self.rpc_backend,
-                    transport_url=self.transport_url)
-
-        if self.rpc_backend:
-            driver.DriverManager = mock.Mock()
-
-            invoke_args = [self.conf,
-                           oslo_messaging.TransportURL.parse(self.conf,
-                                                             self.url)]
-            invoke_kwds = dict(default_exchange='openstack',
-                               allowed_remote_exmods=[])
-
-            driver.DriverManager.side_effect = RuntimeError()
-        try:
-            oslo_messaging.get_transport(self.conf, url=self.url)
-            self.assertFalse(True)
-
-            driver.DriverManager.\
-                assert_called_once_with('oslo.messaging.drivers',
-                                        self.rpc_backend,
-                                        invoke_on_load=True,
-                                        invoke_args=invoke_args,
-                                        invoke_kwds=invoke_kwds)
-        except Exception as ex:
-            ex_cls = self.ex.pop('cls')
-            ex_msg_contains = self.ex.pop('msg_contains')
-
-            self.assertIsInstance(ex, oslo_messaging.MessagingException)
-            self.assertIsInstance(ex, ex_cls)
-            self.assertIn(ex_msg_contains, six.text_type(ex))
-
-            for k, v in self.ex.items():
-                self.assertTrue(hasattr(ex, k))
-                self.assertEqual(v, str(getattr(ex, k)))
+        self.config(transport_url=self.transport_url)
+        ex_cls = self.ex.pop('cls')
+        ex_msg_contains = self.ex.pop('msg_contains')
+        ex = self.assertRaises(
+            ex_cls, oslo_messaging.get_transport, self.conf, url=self.url)
+        self.assertIn(ex_msg_contains, six.text_type(ex))
+        for k, v in self.ex.items():
+            self.assertTrue(hasattr(ex, k))
+            self.assertEqual(v, str(getattr(ex, k)))
 
 
 # FIXME(markmc): this could be used elsewhere
@@ -288,6 +230,7 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
                                                'message',
                                                wait_for_reply=None,
                                                timeout=None,
+                                               call_monitor_timeout=None,
                                                retry=None)
 
     def test_send_all_args(self):
@@ -297,7 +240,8 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
 
         t._send(self._target, 'ctxt', 'message',
                 wait_for_reply='wait_for_reply',
-                timeout='timeout', retry='retry')
+                timeout='timeout', call_monitor_timeout='cm_timeout',
+                retry='retry')
 
         t._driver.send.\
             assert_called_once_with(self._target,
@@ -305,6 +249,7 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
                                     'message',
                                     wait_for_reply='wait_for_reply',
                                     timeout='timeout',
+                                    call_monitor_timeout='cm_timeout',
                                     retry='retry')
 
     def test_send_notification(self):
@@ -351,10 +296,13 @@ class TestTransportUrlCustomisation(test_utils.BaseTestCase):
         def transport_url_parse(url):
             return transport.TransportURL.parse(self.conf, url)
 
-        self.url1 = transport_url_parse("fake://vhost1?x=1&y=2&z=3")
-        self.url2 = transport_url_parse("fake://vhost2?foo=bar")
-        self.url3 = transport_url_parse("fake://vhost1?l=1&l=2&l=3")
-        self.url4 = transport_url_parse("fake://vhost2?d=x:1&d=y:2&d=z:3")
+        self.url1 = transport_url_parse(
+            "fake:/vhost1/localhost:5672/?x=1&y=2&z=3")
+        self.url2 = transport_url_parse("fake:/vhost2/localhost:5672/?foo=bar")
+        self.url3 = transport_url_parse(
+            "fake:/vhost1/localhost:5672/?l=1&l=2&l=3")
+        self.url4 = transport_url_parse(
+            "fake:/vhost2/localhost:5672/?d=x:1&d=y:2&d=z:3")
         self.url5 = transport_url_parse("fake://noport:/?")
 
     def test_hash(self):
