@@ -17,16 +17,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-__all__ = [
-    'DriverLoadFailure',
-    'InvalidTransportURL',
-    'Transport',
-    'TransportHost',
-    'TransportURL',
-    'get_transport',
-    'set_transport_defaults',
-]
-
 import logging
 
 from debtcollector import removals
@@ -38,10 +28,21 @@ from stevedore import driver
 from oslo_messaging._i18n import _LW
 from oslo_messaging import exceptions
 
+__all__ = [
+    'DriverLoadFailure',
+    'InvalidTransportURL',
+    'Transport',
+    'TransportHost',
+    'TransportURL',
+    'get_transport',
+    'set_transport_defaults',
+]
+
 LOG = logging.getLogger(__name__)
 
 _transport_opts = [
     cfg.StrOpt('transport_url',
+               default="rabbit://",
                secret=True,
                help='The network address and optional user credentials for '
                     'connecting to the messaging backend, in URL format. The '
@@ -53,12 +54,6 @@ _transport_opts = [
                     'documentation of oslo_messaging.TransportURL at '
                     'https://docs.openstack.org/oslo.messaging/latest/'
                     'reference/transport.html'),
-    cfg.StrOpt('rpc_backend',
-               deprecated_for_removal=True,
-               deprecated_reason="Replaced by [DEFAULT]/transport_url",
-               default='rabbit',
-               help='The messaging driver to use, defaults to rabbit. Other '
-                    'drivers include amqp and zmq.'),
 
     cfg.StrOpt('control_exchange',
                default='openstack',
@@ -122,13 +117,15 @@ class Transport(object):
         self._driver.require_features(requeue=requeue)
 
     def _send(self, target, ctxt, message, wait_for_reply=None, timeout=None,
-              retry=None):
+              call_monitor_timeout=None, retry=None):
         if not target.topic:
             raise exceptions.InvalidTarget('A topic is required to send',
                                            target)
         return self._driver.send(target, ctxt, message,
                                  wait_for_reply=wait_for_reply,
-                                 timeout=timeout, retry=retry)
+                                 timeout=timeout,
+                                 call_monitor_timeout=call_monitor_timeout,
+                                 retry=retry)
 
     def _send_notification(self, target, ctxt, message, version, retry=None):
         if not target.topic:
@@ -193,13 +190,13 @@ class DriverLoadFailure(exceptions.MessagingException):
         self.ex = ex
 
 
-def _get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None,
+def _get_transport(conf, url=None, allowed_remote_exmods=None,
                    transport_cls=RPCTransport):
     allowed_remote_exmods = allowed_remote_exmods or []
     conf.register_opts(_transport_opts)
 
     if not isinstance(url, TransportURL):
-        url = TransportURL.parse(conf, url, aliases)
+        url = TransportURL.parse(conf, url)
 
     kwargs = dict(default_exchange=conf.control_exchange,
                   allowed_remote_exmods=allowed_remote_exmods)
@@ -219,9 +216,7 @@ def _get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None,
 @removals.remove(
     message='use get_rpc_transport or get_notification_transport'
 )
-@removals.removed_kwarg('aliases',
-                        'Parameter aliases is deprecated for removal.')
-def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
+def get_transport(conf, url=None, allowed_remote_exmods=None):
     """A factory method for Transport objects.
 
     This method will construct a Transport object from transport configuration
@@ -248,11 +243,8 @@ def get_transport(conf, url=None, allowed_remote_exmods=None, aliases=None):
                                   transport will deserialize remote exceptions
                                   from
     :type allowed_remote_exmods: list
-    :param aliases: DEPRECATED: A map of transport alias to transport name
-    :type aliases: dict
     """
-    return _get_transport(conf, url,
-                          allowed_remote_exmods, aliases,
+    return _get_transport(conf, url, allowed_remote_exmods,
                           transport_cls=RPCTransport)
 
 
@@ -334,60 +326,25 @@ class TransportURL(object):
     :type virtual_host: str
     :param hosts: a list of TransportHost objects
     :type hosts: list
-    :param aliases: DEPRECATED: a map of transport alias to transport name
-    :type aliases: dict
     :param query: a dictionary of URL query parameters
     :type query: dict
     """
 
-    @removals.removed_kwarg('aliases',
-                            'Parameter aliases is deprecated for removal.')
     def __init__(self, conf, transport=None, virtual_host=None, hosts=None,
-                 aliases=None, query=None):
+                 query=None):
         self.conf = conf
         self.conf.register_opts(_transport_opts)
-        self._transport = transport
+        self.transport = transport
         self.virtual_host = virtual_host
         if hosts is None:
             self.hosts = []
         else:
             self.hosts = hosts
-        if aliases is None:
-            self.aliases = {}
-        else:
-            self.aliases = aliases
         if query is None:
             self.query = {}
         else:
             self.query = query
 
-        self._deprecation_logged = False
-
-    @property
-    def transport(self):
-        if self._transport is None:
-            transport = self.conf.rpc_backend
-        else:
-            transport = self._transport
-        final_transport = self.aliases.get(transport, transport)
-        if not self._deprecation_logged and final_transport != transport:
-            # NOTE(sileht): The first step is deprecate this one cycle.
-            # To ensure deployer have updated they configuration during Ocata
-            # Then in P we will deprecate aliases kwargs of TransportURL() and
-            # get_transport() for consuming application
-            LOG.warning('legacy "rpc_backend" is deprecated, '
-                        '"%(legacy_transport)s" must be replaced by '
-                        '"%(final_transport)s"' % {
-                            'legacy_transport': transport,
-                            'final_transport': final_transport})
-            self._deprecation_logged = True
-
-        return final_transport
-
-    @transport.setter
-    def transport(self, value):
-        self._transport = value
-
     def __hash__(self):
         return hash((tuple(self.hosts), self.transport, self.virtual_host))
 
@@ -450,10 +407,8 @@ class TransportURL(object):
 
         return url
 
-    @removals.removed_kwarg('aliases',
-                            'Parameter aliases is deprecated for removal.')
     @classmethod
-    def parse(cls, conf, url=None, aliases=None):
+    def parse(cls, conf, url=None):
         """Parse a URL as defined by :py:class:`TransportURL` and return a
         TransportURL object.
 
@@ -483,16 +438,12 @@ class TransportURL(object):
         :type conf: oslo.config.cfg.ConfigOpts
         :param url: The URL to parse
         :type url: str
-        :param aliases: A map of transport alias to transport name
-        :type aliases: dict
         :returns: A TransportURL
         """
 
         if not url:
             conf.register_opts(_transport_opts)
         url = url or conf.transport_url
-        if not url:
-            return cls(conf) if aliases is None else cls(conf, aliases=aliases)
 
         if not isinstance(url, six.string_types):
             raise InvalidTransportURL(url, 'Wrong URL type')
@@ -576,7 +527,4 @@ class TransportURL(object):
                         {'hosts_with_credentials': hosts_with_credentials,
                          'hosts_without_credentials':
                          hosts_without_credentials})
-        if aliases is None:
-            return cls(conf, transport, virtual_host, hosts, query=query)
-        else:
-            return cls(conf, transport, virtual_host, hosts, aliases, query)
+        return cls(conf, transport, virtual_host, hosts, query)
