@@ -10,8 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import eventlet
-eventlet.monkey_patch()
+import eventlet  # noqa
+eventlet.monkey_patch()  # noqa
 
 import argparse
 import bisect
@@ -457,7 +457,7 @@ class RPCClient(Client):
 
 class NotifyClient(Client):
     def __init__(self, client_id, transport, topic, wait_after_msg):
-        client = notify.Notifier(transport, driver='messaging', topic=topic)
+        client = notify.Notifier(transport, driver='messaging', topics=topic)
         client = client.prepare(publisher_id='publisher-%d' % client_id)
         method = _notify
         super(NotifyClient, self).__init__(client_id, client, method,
